/**
 * 
 */
package org.ajax4jsf.portlet.application;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import javax.faces.FacesException;
import javax.faces.application.FacesMessage;
import javax.faces.component.UIComponent;
import javax.faces.component.UIViewRoot;
import javax.faces.context.FacesContext;
import javax.portlet.ActionRequest;
import javax.portlet.RenderRequest;

/**
 * @author asmirnov
 * 
 */
public class PortletViewState implements Serializable {

    /**
         * 
         */
    private static final long serialVersionUID = 5630637804542426709L;

    public static final String SAVED_VIEW_ATTRIBUTE = PortletViewState.class
	    .getName();

    private Map messages = new HashMap();

    private Object treeStructure;

    private Object componentsState;

    private UIViewRoot viewRoot;

    public void save(FacesContext facesContext) {
	UIViewRoot viewRoot = facesContext.getViewRoot();
	// setViewRoot(viewRoot);
	TreeStrutureNode rootNode = new TreeStrutureNode();
	rootNode.apply(viewRoot);
	setTreeStructure(rootNode);
	setComponentsState(viewRoot.processSaveState(facesContext));
	saveMessages(facesContext);
	// facesContext.getExternalContext().getSessionMap().put(PortletViewState.SAVED_VIEW_ATTRIBUTE,
	// this);
    }

    /**
         * @param facesContext
         */
    public void saveMessages(FacesContext facesContext) {
	Iterator idsWithMessages = facesContext.getClientIdsWithMessages();
	while (idsWithMessages.hasNext()) {
	    String id = (String) idsWithMessages.next();
	    Iterator messages = facesContext.getMessages(id);
	    while (messages.hasNext()) {
		FacesMessage message = (FacesMessage) messages.next();
		addMessage(id, message);
	    }
	}
    }

    public UIViewRoot restore(FacesContext facesContext,ComponentsLoader loader) {
	// facesContext.setViewRoot(getViewRoot());
	TreeStrutureNode structure = (TreeStrutureNode) getTreeStructure();
	UIViewRoot root = null;
	if (null != structure) {
	    root = (UIViewRoot) structure.restore(loader);
	    root.processRestoreState(facesContext, getComponentsState());
	    // facesContext.setViewRoot(root);
	    if (facesContext.getExternalContext().getRequest() instanceof RenderRequest) {
		restoreMessages(facesContext);

	    }

	}
	return root;
    }

    /**
         * @param facesContext
         */
    public void restoreMessages(FacesContext facesContext) {
	Iterator idsWithMessages = getClientIdsWithMessages();
	while (idsWithMessages.hasNext()) {
	    String id = (String) idsWithMessages.next();
	    Iterator messages = getMessages(id);
	    while (messages.hasNext()) {
		FacesMessage message = (FacesMessage) messages.next();
		facesContext.addMessage(id, message);
	    }
	}
    }

    /**
         * @return the _messages
         */
    public Map getMessages() {
	return messages;
    }

    /**
         * @param _messages
         *                the _messages to set
         */
    public void setMessages(Map _messages) {
	this.messages = _messages;
    }

    /**
         * @return the _viewRoot
         */
    public UIViewRoot getViewRoot() {
	return viewRoot;
    }

    /**
         * @param root
         *                the _viewRoot to set
         */
    public void setViewRoot(UIViewRoot root) {
	viewRoot = root;
    }

    public void addMessage(String clientId, FacesMessage message) {
	List list = (List) messages.get(clientId);
	if (list == null) {
	    list = new ArrayList();
	    messages.put(clientId, list);
	}
	list.add(message);
    }

    public Iterator getClientIdsWithMessages() {
	return (messages.keySet().iterator());
    }

    public Iterator getMessages(String clientId) {
	List list = (List) messages.get(clientId);
	if (list != null) {
	    return (list.iterator());
	} else {
	    return (Collections.EMPTY_LIST.iterator());
	}
    }

    /**
         * @return the componentsState
         */
    public Object getComponentsState() {
	return componentsState;
    }

    /**
         * @param componentsState
         *                the componentsState to set
         */
    public void setComponentsState(Object componentsState) {
	this.componentsState = componentsState;
    }

    /**
         * @return the treeStructure
         */
    public Object getTreeStructure() {
	return treeStructure;
    }

    /**
         * @param treeStructure
         *                the treeStructure to set
         */
    public void setTreeStructure(Object treeStructure) {
	this.treeStructure = treeStructure;
    }

    protected static class TreeStrutureNode implements Serializable {
	/**
         * 
         */
	private static final long serialVersionUID = -9038742487716977254L;

	private Map facets = new HashMap();

	private List children = new ArrayList();

	private String type;

	private String id;

	public void apply(UIComponent component) {
	    type = component.getClass().getName();
	    id = component.getId();
	    Map componentFacets = component.getFacets();
	    for (Iterator i = componentFacets.entrySet().iterator(); i
		    .hasNext();) {
		Entry element = (Entry) i.next();
		UIComponent f = (UIComponent) element.getValue();
		if (!f.isTransient()) {
		    TreeStrutureNode facet = new TreeStrutureNode();
		    facet.apply(f);
		    facets.put(element.getKey(), facet);

		}
	    }
	    for (Iterator i = component.getChildren().iterator(); i.hasNext();) {
		UIComponent child = (UIComponent) i.next();
		if (!child.isTransient()) {
		    TreeStrutureNode t = new TreeStrutureNode();
		    t.apply(child);
		    children.add(t);

		}
	    }
	}

	public UIComponent restore(ComponentsLoader loader) {
	    UIComponent component;
		component = loader.createComponent(type);
		component.setId(id);
	    for (Iterator i = facets.entrySet().iterator(); i.hasNext();) {
		Entry element = (Entry) i.next();
		UIComponent facet = ((TreeStrutureNode) element.getValue())
			.restore(loader);
		component.getFacets().put((String) element.getKey(), facet);
	    }
	    for (Iterator i = children.iterator(); i.hasNext();) {
		TreeStrutureNode node = (TreeStrutureNode) i.next();
		UIComponent child = node.restore(loader);
		component.getChildren().add(child);
	    }
	    return component;
	}
    }
}
