package org.richfaces.model.selection;

import junit.framework.TestCase;

public class ClientSelectionTest extends TestCase {

	
	private ClientSelection clientSelection;

	public ClientSelectionTest(String name) {
		super(name);
	}

	protected void setUp() throws Exception {
		super.setUp();
		clientSelection = new ClientSelection();
	}

	protected void tearDown() throws Exception {
		clientSelection = null;
		super.tearDown();
	}

	public void testAddRange() {
		SelectionRange range = new SelectionRange(10, 20);
		clientSelection.addRange(range);
		assertTrue(clientSelection.isSelected(10));
		assertTrue(clientSelection.isSelected(11));
		assertTrue(clientSelection.isSelected(15));
		assertTrue(clientSelection.isSelected(20));
		assertFalse(clientSelection.isSelected(0));
		assertFalse(clientSelection.isSelected(30));
	}

	public void testIsSelected() {
		SelectionRange range = new SelectionRange(10, 20);
		clientSelection.addRange(range);
		assertTrue(clientSelection.isSelected(10));
		assertTrue(clientSelection.isSelected(11));
		assertTrue(clientSelection.isSelected(15));
		assertTrue(clientSelection.isSelected(20));
		assertFalse(clientSelection.isSelected(0));
		assertFalse(clientSelection.isSelected(30));
	}

	public void testGetRanges() {
		SelectionRange range = new SelectionRange(10, 20);
		clientSelection.addRange(range);
		range = new SelectionRange(30, 40);
		clientSelection.addRange(range);
		
		assertNotNull(clientSelection.getRanges());
		assertEquals(2, clientSelection.getRanges().size());
	}

	public void testAddIndex() {
		clientSelection.addRange(new SelectionRange(10, 20));
		clientSelection.addIndex(21);
		assertEquals(1, clientSelection.getRanges().size());
		assertTrue(clientSelection.isSelected(21));
		clientSelection.addRange(new SelectionRange(23, 30));
		assertEquals(2, clientSelection.getRanges().size());
		assertFalse(clientSelection.isSelected(22));
		assertTrue(clientSelection.isSelected(23));
		
		clientSelection.addIndex(22);
		assertTrue(clientSelection.isSelected(22));
		assertEquals(1, clientSelection.getRanges().size());
		
		
	}

	public void testSetSelectionFlag() {
		clientSelection.setSelectionFlag(null);
		assertFalse(clientSelection.isReset());
		assertFalse(clientSelection.isSelectAll());

		clientSelection.setSelectionFlag(ClientSelection.FLAG_ALL);
		assertFalse(clientSelection.isReset());
		assertTrue(clientSelection.isSelectAll());

		clientSelection.setSelectionFlag(ClientSelection.FLAG_RESET);
		assertTrue(clientSelection.isReset());
		assertFalse(clientSelection.isSelectAll());

		clientSelection.setSelectionFlag("zzzzzz");
		assertFalse(clientSelection.isReset());
		assertFalse(clientSelection.isSelectAll());

	}
	
	
}
